﻿using System.Collections.Generic;
using System.Linq.Expressions;
using System.Web;
using System.IO;
using System.Web.Routing;
using Resources;

namespace System.Web.Mvc
{
    public static class ControllerExtensions
    {
        #region ExecuteSendFile extension : download attachments by CLT-Services (http://www.clt-services.com/)
        public delegate void FileWriterDelegate(string filename, Stream outputStream);

        private static void ExecuteSendFile(HttpContextBase context, string filename, FileWriterDelegate fileWriter)
        {
            context.Response.ClearHeaders();
            context.Response.ClearContent();
            context.Response.AppendHeader("Content-disposition", 
                string.Format("attachment;filename=\"{0}\"", 
                HttpUtility.UrlPathEncode(Path.GetFileName(filename))));
            if (fileWriter != null)
                fileWriter(filename, context.Response.OutputStream);
            else
            {
                if(File.Exists(filename))
                    context.Response.WriteFile(filename);
                else
                    throw new HttpException(404, Common.FileNotFound);
            }            
        }

        public static ActionResult SendFileAsAttachment(this Controller controller, string filename)
        {
            return new DelegatedActionResult(context => ExecuteSendFile(context.HttpContext, filename, null));
        }

        public static ActionResult SendFileAsAttachment(this Controller controller, string filename, FileWriterDelegate fileWriter)
        {
            return new DelegatedActionResult(context => ExecuteSendFile(context.HttpContext, filename, fileWriter));
        }
        #endregion
        #region RedirectToAction extensions by eWorld.UI : http://blog.eworldui.net/post/2008/05/ASPNET-MVC-Expression-Based-RedirectToAction-Method.aspx

        public static ActionRedirectResult RedirectToAction<T>(this Controller controller, Expression<Action<T>> action)
            where T: Controller
        {
            return RedirectToAction(controller, action, null);
        }

        public static ActionRedirectResult RedirectToAction<T>(this Controller controller, Expression<Action<T>> action, object values)
            where T: Controller
        {
            return RedirectToAction(controller, action, new RouteValueDictionary(values));
        }

        public static ActionRedirectResult RedirectToAction<T>(this Controller controller, Expression<Action<T>> action, RouteValueDictionary values)
            where T: Controller
        {
            var body = action.Body as MethodCallExpression;
            if (body == null)
                throw new InvalidOperationException("Expression must be a method call.");

            if (body.Object != action.Parameters[0])
                throw new InvalidOperationException("Method call must target lambda argument.");

            string actionName = body.Method.Name;
            string controllerName = typeof(T).Name;

            if (controllerName.EndsWith("Controller", StringComparison.OrdinalIgnoreCase))
                controllerName = controllerName.Remove(controllerName.Length - 10, 10);

            RouteValueDictionary parameters = LinkBuilder.BuildParameterValuesFromExpression(body);

            values = values ?? new RouteValueDictionary();
            values.Add("controller", controllerName);
            values.Add("action", actionName);

            if (parameters != null)
            {
                foreach (var parameter in parameters)
                {
                    values.Add(parameter.Key, parameter.Value);
                }
            }

            return new ActionRedirectResult(values);
        }
        #endregion
    }
}
